package com.supermap.samplecode.mapping;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Toolkit;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;

import com.supermap.data.Workspace;
import com.supermap.ui.GeometrySelectedEvent;
import com.supermap.ui.GeometrySelectedListener;
import com.supermap.ui.MapControl;

/**
 * <p>
 * Title:选择集管理
 * Title:Selection Display
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap Objects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
 * ------------------------------CopyRight----------------------------
 *This file contants the sample code of SuperMap Objects Java.
 * Copyright @2005-2012 SuperMap Software Co., Ltd. All Rights Reserved. 
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：选择集风格、选择对象属性查看、指定对象为选中状态
 * 2、示例数据：安装目录\SampleData\City\Changchun.smwu
 * 3、关键类型/成员: 
 * 		DatasetVector.query 方法
 * 		DatasetVector.getRecordset 方法
 *      Selection.getDataset 方法
 *      Selection.getStyle 方法
 *      Selection.getCount 方法
 *      Selection.get 方法
 *      Selection.fromRecordset 方法
 *      Recordset.getFieldCount 方法
 *      Recordset.getRecordCount 方法
 *      Recordset.getFieldValue 方法
 *      Recordset.moveNext 方法
 *      Recordset.dispose 方法
 *      GeoStyle.setFillForeColor 方法
 *      GeoStyle.setFillBackOpaque 方法
 *      GeoStyle.setFillBackColor 方法
 *      GeoStyle.setFillSymbolID 方法
 *      GeoStyle.setFillOpaqueRate 方法
 *      GeoStyle.setLineColor 方法
 *      GeoStyle.setLineWidth 方法
 * 4、使用步骤：
 *  (1)点击浏览属性，在下方属性表中浏览全部属性记录
 *  (2)点击图层中的几何对象为选中状态，在下方属性表中显示该几何对象的属性（可以框选）
 *  (3)点击下方属性表中的记录，图层中对应的几何对象成为选中状态（可以多选）
 *  (4)勾选显示风格框后，按照特定的风格进行显示所选中的对象，去掉勾选，按照默认的风格进行显示
 * ------------------------------------------------------------------------------
 * ---------------------SuperMap Objects Java Sample Code Description------------------------
 * 
 * 1. Sample Code Description:Selection style,Browse the attribute,Set the item selected.
 * 2. Sample Data Location:Installation directory\SampleData\City\changchun.smwu
 * 3. Key Classes/Methods:
 *     
 * 		DatasetVector.query method
 * 		DatasetVector.getRecordset method
 *      Selection.getDataset method
 *      Selection.getStyle method
 *      Selection.getCount method
 *      Selection.get method
 *      Selection.fromRecordset method
 *      Recordset.getFieldCount method
 *      Recordset.getRecordCount method
 *      Recordset.getFieldValue method
 *      Recordset.moveNext method
 *      Recordset.dispose method
 *      GeoStyle.setFillForeColor method
 *      GeoStyle.setFillBackOpaque method
 *      GeoStyle.setFillBackColor method
 *      GeoStyle.setFillSymbolID method
 *      GeoStyle.setFillOpaqueRate method
 *      GeoStyle.setLineColor method
 *      GeoStyle.setLineWidth method
 * 4,Procedures:
 *  (1)Browse the attribute.
 *  (2)Set the Geometry object in the layer selected.
 *  (3)Click the attribute, and the object is selected.
 *  (4)Select the Style box, and display the object with the specified style.
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * Company: SuperMap Software Co., Ltd
 * </p>
 * 
 */

public class MainFrame extends JFrame {

	private static final long serialVersionUID = 1L;

	private Workspace m_workspace;

	private MapControl m_mapControl;

	JTable m_tableProperties;

	JScrollPane m_scrollPane;

	JCheckBox m_checkBoxStyle;

	JButton m_buttonProperties;

	JPanel m_panel;;

	private SampleRun m_sampleRun;

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				MainFrame mainFrame = new MainFrame();
				mainFrame.setVisible(true);
				mainFrame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
			}
		});

	}

	public MainFrame() {
		try {
	/**
	 * 初始化窗体
	 * Initialize the forms.
	 */
			initialize();
		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	private void initialize() throws Exception {

		// 设置主窗体的标题、大小及位置
		// Set the forms.
		Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
		this.setExtendedState(JFrame.MAXIMIZED_BOTH);
		this.setSize(960, 600);
		this.setLocation((screenSize.width - this.getSize().width) / 2,
				(screenSize.height - this.getSize().height) / 2);
		this.setTitle(getText("this"));
		this.getContentPane().setLayout(new BorderLayout());

		this.addWindowListener(new WindowAdapter() {
			// 在主窗体加载时，初始化SampleRun类型，来完成功能的展现
			// Initialize a SampleRun object when loading the main forms.
			public void windowOpened(WindowEvent e) {
				m_workspace = new Workspace();
				// 实例化m_sampleRun
				// Initialize m_sampleRun.
				m_sampleRun = new SampleRun(m_workspace, m_mapControl,
						m_tableProperties, m_checkBoxStyle);
			}

			// 在窗体关闭时，需要释放相关的资源
			// Dispose the resources when closing the forms.
			public void windowClosing(WindowEvent e) {
				m_mapControl.dispose();
				m_workspace.dispose();
			}
		});
		m_mapControl = new MapControl();
		// 初始化控件
		// Initialize.
		m_tableProperties = new JTable() {
			private static final long serialVersionUID = 1L;

			public boolean isCellEditable(int row, int column) {
				return false;
			}
		};
		m_tableProperties.getTableHeader().setReorderingAllowed(false);
		m_scrollPane = new JScrollPane();
		m_checkBoxStyle = new JCheckBox();
		m_buttonProperties = new JButton();
		m_panel = new JPanel();

		// 设置m_buttonProperties
		// Set the m_buttonProperties 
		m_buttonProperties.setText(getText("m_buttonProperties"));
		m_buttonProperties.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				m_sampleRun.getProperties();
			}
		});

		// 设置m_checkBoxStyle
		//Set the m_checkBoxStyle
		m_checkBoxStyle.setText(getText("m_checkBoxStyle"));
		m_checkBoxStyle.addItemListener(new ItemListener() {
			public void itemStateChanged(ItemEvent e) {
				m_sampleRun.configStyle();
			}
		});

		this.getContentPane().add(m_panel, BorderLayout.NORTH);
		FlowLayout flowLayout = new FlowLayout();
		flowLayout.setAlignment(FlowLayout.LEFT);
		m_panel.setLayout(flowLayout);
		m_panel.add(m_buttonProperties);
		m_panel.add(m_checkBoxStyle);

		// 设置m_mapControl	
		// Set the m_mapControl		
		this.getContentPane().add(m_mapControl, BorderLayout.CENTER);
		m_mapControl
				.addGeometrySelectedListener(new GeometrySelectedListener() {
					public void geometrySelected(GeometrySelectedEvent e) {
						m_sampleRun.showProperties();
					}
				});

		// 设置m_scrollPane和m_tableProperties
		// Set the m_scrollPane and m_tableProperties.
		this.getContentPane().add(m_scrollPane, java.awt.BorderLayout.SOUTH);
		m_scrollPane.setPreferredSize(new Dimension(960, 200));
		m_scrollPane.getViewport().add(m_tableProperties);
		m_tableProperties.addMouseListener(new MouseAdapter() {
			public void mouseReleased(MouseEvent e) {
				m_sampleRun.getGeometries();
			}
		});

	}
	
	private String getText(String id)
	{
		String text = "";
		if (com.supermap.data.Environment.getCurrentCulture().contentEquals("zh-CN"))
        {  
			if(id == "this")
			{
				text = "选择集管理";
			}
			if(id == "m_buttonProperties")
			{
				text = "对象属性浏览";
			}
			if(id == "m_checkBoxStyle")
			{
				text = "自定义选择集风格";
			}
		}
		else
		{
			if(id == "this")
			{
				text = "Selection Display";
			}
			if(id == "m_buttonProperties")
			{
				text = "Browse the attribute";
			}
			if(id == "m_checkBoxStyle")
			{
				text = "Custom style";
			}
		}
		return text;
	}

}

