package com.supermap.samplecode.mapping;

import java.awt.Color;

import javax.swing.JCheckBox;
import javax.swing.JTable;
import javax.swing.table.DefaultTableModel;

import com.supermap.data.CursorType;
import com.supermap.data.DatasetVector;
import com.supermap.data.Datasets;
import com.supermap.data.Datasource;
import com.supermap.data.Datasources;
import com.supermap.data.GeoStyle;
import com.supermap.data.Recordset;
import com.supermap.data.Workspace;
import com.supermap.data.WorkspaceConnectionInfo;
import com.supermap.data.WorkspaceType;
import com.supermap.mapping.Map;
import com.supermap.mapping.Selection;
import com.supermap.mapping.StyleOptions;
import com.supermap.ui.MapControl;

/**
 * <p>
 * Title:选择集管理
 * Title:Selection Display
 * </p>
 * 
 * <p>
 * Description:
 * ============================================================================>
 * ------------------------------版权声明----------------------------
 * 此文件为SuperMap Objects Java 的示范代码 
 * 版权所有：北京超图软件股份有限公司
 * ----------------------------------------------------------------
  * ------------------------------CopyRight----------------------------
 *This file contants the sample code of SuperMap Objects Java.
 * Copyright @2005-2012 SuperMap Software Co., Ltd. All Rights Reserved. 
 * ----------------------------------------------------------------
 * ---------------------SuperMap iObjects Java 示范程序说明------------------------
 * 
 * 1、范例简介：选择集风格、选择对象属性查看、指定对象为选中状态
 * 2、示例数据：安装目录\SampleData\City\Changchun.smwu
 * 3、关键类型/成员: 
 * 		DatasetVector.query 方法
 * 		DatasetVector.getRecordset 方法
 *      Selection.getDataset 方法
 *      Selection.getStyle 方法
 *      Selection.getCount 方法
 *      Selection.get 方法
 *      Selection.fromRecordset 方法
 *      Recordset.getFieldCount 方法
 *      Recordset.getRecordCount 方法
 *      Recordset.getFieldValue 方法
 *      Recordset.moveNext 方法
 *      Recordset.dispose 方法
 *      GeoStyle.setFillForeColor 方法
 *      GeoStyle.setFillBackOpaque 方法
 *      GeoStyle.setFillBackColor 方法
 *      GeoStyle.setFillSymbolID 方法
 *      GeoStyle.setFillOpaqueRate 方法
 *      GeoStyle.setLineColor 方法
 *      GeoStyle.setLineWidth 方法
 * 4、使用步骤：
 *  (1)点击浏览属性，在下方属性表中浏览全部属性记录
 *  (2)点击图层中的几何对象为选中状态，在下方属性表中显示该几何对象的属性（可以框选）
 *  (3)点击下方属性表中的记录，图层中对应的几何对象成为选中状态（可以多选）
 *  (4)勾选显示风格框后，按照特定的风格进行显示所选中的对象，去掉勾选，按照默认的风格进行显示
 * ------------------------------------------------------------------------------
 * ---------------------SuperMap Objects Java Sample Code Description------------------------
 * 
 * 1. Sample Code Description:Selection style,Browse the attribute,Set the item selected.
 * 2. Sample Data Location:Installation directory\SampleData\City\changchun.smwu
 * 3. Key Classes/Methods:
 *     
 * 		DatasetVector.query method
 * 		DatasetVector.getRecordset method
 *      Selection.getDataset method
 *      Selection.getStyle method
 *      Selection.getCount method
 *      Selection.get method
 *      Selection.fromRecordset method
 *      Recordset.getFieldCount method
 *      Recordset.getRecordCount method
 *      Recordset.getFieldValue method
 *      Recordset.moveNext method
 *      Recordset.dispose method
 *      GeoStyle.setFillForeColor method
 *      GeoStyle.setFillBackOpaque method
 *      GeoStyle.setFillBackColor method
 *      GeoStyle.setFillSymbolID method
 *      GeoStyle.setFillOpaqueRate method
 *      GeoStyle.setLineColor method
 *      GeoStyle.setLineWidth method
 * 4,Procedures:
 *  (1)Browse the attribute.
 *  (2)Set the Geometry object in the layer selected.
 *  (3)Click the attribute, and the object is selected.
 *  (4)Select the Style box, and display the object with the specified style.
 * ------------------------------------------------------------------------------
 * ============================================================================>
 * </p> 
 * 
 * <p>
 * Company: 北京超图软件股份有限公司
 * Company: SuperMap Software Co., Ltd
 * </p>
 * 
 */

public class SampleRun {

	private Workspace m_workspace;

	private MapControl m_mapControl;

	private JTable m_tableProperties;

	private JCheckBox m_checkBoxStyle;

	private GeoStyle m_geoStyle;

	private static String m_datasetVectorName = "WaterPoly";

	public SampleRun(Workspace workspace, MapControl mapControl, JTable table,
			JCheckBox checkBox) {
		this.m_workspace = workspace;
		this.m_mapControl = mapControl;
		this.m_tableProperties = table;
		this.m_checkBoxStyle = checkBox;
		initialize();
	}

	/**
	 * 获得数据集的全部属性，在表中显示
	 * Get and display the attribute.
	 * 
	 */
	public void getProperties() {
		DatasetVector datasetVector = getDatasetVector(m_datasetVectorName);
		Recordset recordset = datasetVector.getRecordset(false,
				CursorType.STATIC);
		getProperties(recordset);
		recordset.dispose();
	}

	/**
	 * 获得几何对象
	 * Get the Geometry object.
	 * 
	 */
	public void getGeometries() {

		// 获得属性表中所选的记录，得到相应的smid数组
		// Get the record in the table.
		int[] selectedRows = this.m_tableProperties.getSelectedRows();
		int length = selectedRows.length;
		int[] ids = new int[length];
		for (int i = 0; i < selectedRows.length; i++) {
			ids[i] = (Integer) m_tableProperties.getValueAt(selectedRows[i], 0);
		}

		// 获得图层的选择集对象
		// Get the selection.
		Map map = m_mapControl.getMap();
		Selection selection = map.getLayers().get(0).getSelection();

		// 获得选择集所属的数据集，并按照smid数组查询得到记录集
		// Get the dataset, and get the recordset.
		DatasetVector datasetVector = selection.getDataset();
		Recordset recordset = datasetVector.query(ids, CursorType.STATIC);

		// 将记录集转化为选择集，即将相应的几何对象变为选中状态
		// Convert the recordset to selection.
		selection.fromRecordset(recordset);

		// 刷新地图，释放记录集
		// Refresh map, and dispose the recordset.
		map.refresh();
		recordset.dispose();

	}

	/**
	 * 配置选择集显示风格，勾选为特定风格，不勾选为默认
	 * Set the style of the selection.
	 */
	public void configStyle() {
		Map map = m_mapControl.getMap();
		Selection selection = map.getLayers().get(0).getSelection();
		
		selection.setStyleOptions(StyleOptions.FILLBACKCOLOR, true);
		selection.setStyleOptions(StyleOptions.FILLBACKOPAQUE, true);
		selection.setStyleOptions(StyleOptions.FILLFORECOLOR, true);
		selection.setStyleOptions(StyleOptions.FILLGRADIENTANGLE, true);
		selection.setStyleOptions(StyleOptions.FILLGRADIENTMODE, true);
		selection.setStyleOptions(StyleOptions.FILLGRADIENTOFFSETRATIOX, true);
		selection.setStyleOptions(StyleOptions.FILLGRADIENTOFFSETRATIOY, true);
		selection.setStyleOptions(StyleOptions.FILLOPAQUERATE, true);
		selection.setStyleOptions(StyleOptions.FILLSYMBOLID, true);
		selection.setStyleOptions(StyleOptions.LINECOLOR, true);
		selection.setStyleOptions(StyleOptions.LINESYMBOLID, true);
		selection.setStyleOptions(StyleOptions.LINEWIDTH, true);
		selection.setStyleOptions(StyleOptions.MARKERANGLE, true);
		selection.setStyleOptions(StyleOptions.MARKERSIZE, true);
		selection.setStyleOptions(StyleOptions.MARKERSYMBOLID, true);
		
		m_geoStyle = selection.getStyle();
		if (m_checkBoxStyle.isSelected()) {
			selection.setDefaultStyleEnabled(false);
			m_geoStyle.setFillForeColor(Color.YELLOW);
			m_geoStyle.setFillBackOpaque(false);
			m_geoStyle.setFillBackColor(Color.BLUE);
			m_geoStyle.setFillSymbolID(0);
			m_geoStyle.setFillOpaqueRate(25);
			m_geoStyle.setLineColor(Color.RED);
			m_geoStyle.setLineWidth(1.0);
		} else {
			selection.setDefaultStyleEnabled(true);
		}
		map.refresh();
	}

	/**
	 * 选择对象属性查看
	 * Browse the selected object.
	 * 
	 */
	public void showProperties() {

		// 获得图层的选择集，得到几何对象个数，得到相应的smid数组
		// Get the layer selection, and get the count of the objects.
		Map map = m_mapControl.getMap();
		Selection selection = map.getLayers().get(0).getSelection();
		int count = selection.getCount();
		int[] ids = new int[count];
		for (int i = 0; i < count; i++) {
			ids[i] = selection.get(i);
		}

		// 按照smid数组查询数据集得到记录集，显示在属性表中
		// Get and display the recordset.
		DatasetVector datasetVector = selection.getDataset();
		Recordset recordset = datasetVector.query(ids, CursorType.STATIC);
		getProperties(recordset);
		recordset.dispose();
	}

	/**
	 * 打开需要的工作空间文件及地图
	 * Open the workspace and map.
	 */
	private void initialize() {

		try {
			// 打开工作空间
			// Open workspace
			WorkspaceConnectionInfo workspaceConnectionInfo = new WorkspaceConnectionInfo();
			workspaceConnectionInfo.setType(WorkspaceType.SMWU);
			workspaceConnectionInfo
					.setServer("../../SampleData/City/Changchun.smwu");
			m_workspace.open(workspaceConnectionInfo);

			// 获得数据集，将其添加到MapControl的图层中
			// Get the dataset.
			DatasetVector datasetVector = getDatasetVector(m_datasetVectorName);
			Map map = m_mapControl.getMap();
			map.setWorkspace(m_workspace);
			map.getLayers().add(datasetVector, true);
			map.refresh();


		} catch (Exception ex) {
			System.out.println(ex.getMessage());
		}
	}

	/**
	 * 通过数据集名称获得数据集
	 * Get the dataset by the name
	 * 
	 * @param name
	 *            要获得的数据集名称
	 * @return 获得的数据集
	 * @param name The specified name
	 * @return The specified dataset.
	 */
	private DatasetVector getDatasetVector(String name) {
		Datasources datasources = m_workspace.getDatasources();
		Datasource datasource = datasources.get(0);
		Datasets datasets = datasource.getDatasets();
		DatasetVector datasetVector = (DatasetVector) datasets.get(name);
		return datasetVector;
	}

	/**
	 * 获得记录集的属性，在表中显示
	 * Get and display the attribute.
	 * 
	 * @param recordset
	 *            获得属性的记录集
	  @param recordset The specified recordset.
	 */
	private void getProperties(Recordset recordset) {
		int fieldcount = recordset.getFieldCount();
		int recordcount = recordset.getRecordCount();
		Object[][] tableValues = new Object[recordcount][fieldcount];
		Object[] tableColumns = new Object[fieldcount];
		for (int j = 0; j < fieldcount; j++) {
			tableColumns[j] = recordset.getFieldInfos().get(j).getName();
		}
		for (int i = 0; i < recordcount; i++) {
			for (int j = 0; j < fieldcount; j++) {
				tableValues[i][j] = recordset.getFieldValue(j);
			}
			recordset.moveNext();
		}
		DefaultTableModel tableModel = new DefaultTableModel(tableValues,
				tableColumns);
		this.m_tableProperties.setModel(tableModel);

	}

}

